/* 06/01/2010 - Changed to use Microchip_USB_2_7 folder and USB stack */
/* 08/28/2010 - Changed to use Microchip folder - from Microchip Applications Library USB
				stack v2.7 - updated project to use unchanged MAL Microchip folder - no code changes */
/* 09/02/2011 - Tested with Microchip USB Stack v2.9a, C32 v2.01, MPLAB v8.76 - no code changes */



#include "GenericTypeDefs.h"
#include "Compiler.h"
#include "usb_config.h"
#include "USB/usb.h"
#include "USB/usb_function_cdc.h"
#include "HardwareProfile.h"
#include "HelloUSBWorld.h"

// Let compile time pre-processor calculate the CORE_TICK_PERIOD
#define SYS_FREQ 				(80000000L)
#define TOGGLES_PER_SEC			1000
#define CORE_TICK_RATE	       (SYS_FREQ/2/TOGGLES_PER_SEC)

/** P R I V A T E  P R O T O T Y P E S ***************************************/

/** P R I V A T E  V A R I A B L E S *****************************************/
char USB_In_Buffer[64];
char USB_Out_Buffer[64];

// Decriments every 1 ms.
volatile static unsigned int OneMSTimer;

/** D E C L A R A T I O N S **************************************************/

/******************************************************************************
 * Function:        void UserInit(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This routine should take care of all of the demo code
 *                  initialization that is required.
 *
 * Note:            
 *
 *****************************************************************************/
void UserInit(void)
{
    //Initialize all of the LED pins
	mInitAllLEDs();

	// Open up the core timer at our 1ms rate
	OpenCoreTimer(CORE_TICK_RATE);

    // set up the core timer interrupt with a prioirty of 2 and zero sub-priority
	mConfigIntCoreTimer((CT_INT_ON | CT_INT_PRIOR_2 | CT_INT_SUB_PRIOR_0));

    // enable multi-vector interrupts
	INTEnableSystemMultiVectoredInt();

}//end UserInit

/********************************************************************
 * Function:        void ProcessIO(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This function is a place holder for other user
 *                  routines. It is a mixture of both USB and
 *                  non-USB tasks.
 *
 * Note:            None
 *******************************************************************/
void ProcessIO(void)
{
	unsigned char numBytesRead = 0;

    //Blink the LEDs according to the USB device status
    BlinkUSBStatus();

    // User Application USB tasks
    if (
    	(USBDeviceState < CONFIGURED_STATE)
    	||
    	(USBSuspendControl == 1)
    )
    {
	    return;
	}

	// Pull in some new data if there is new data to pull in
	numBytesRead = getsUSBUSART (USB_In_Buffer,64);

	if (numBytesRead != 0)
	{
		sprintf (USB_Out_Buffer, "Received USB bytes. Hello!\r\n");
		putUSBUSART (USB_Out_Buffer, strlen (USB_Out_Buffer));
		mLED_2_Toggle();
		mLED_3_On();
		OneMSTimer = 1000;
	}
	
	if (!OneMSTimer)
	{
		mLED_3_Off();
	}

    CDCTxService();
}//end ProcessIO

void __ISR(_CORE_TIMER_VECTOR, ipl2) CoreTimerHandler(void)
{
    // clear the interrupt flag
    mCTClearIntFlag();

	if (OneMSTimer)
	{
		OneMSTimer--;
	}

    // update the period
    UpdateCoreTimer(CORE_TICK_RATE);
}
